package net.sourceforge.openutils.notifications.messages;

import java.io.IOException;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.mail.internet.MimeBodyPart;

import net.sourceforge.openutils.notifications.NotificationException;

import org.apache.commons.lang.StringUtils;

import freemarker.cache.ClassTemplateLoader;
import freemarker.ext.beans.BeanModel;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateMethodModelEx;
import freemarker.template.TemplateModelException;


/**
 * @author molaschi
 * @version $Id: $
 */
public abstract class FreemarkerEmailNotification extends EmailNotification
{

    private Configuration configuration;

    private Map<String, Object> data;

    protected String body;

    public Locale getLocale()
    {
        return Locale.ENGLISH;
    }

    /**
     * Sets the configuration.
     * @param configuration the configuration to set
     */
    public void setConfiguration(Configuration configuration)
    {
        this.configuration = configuration;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getMessageBody() throws NotificationException
    {
        if (body == null)
        {
            if (data == null)
            {
                data = new HashMap<String, Object>();
            }
            for (Method m : this.getClass().getDeclaredMethods())
            {
                if ((m.getModifiers() & Method.PUBLIC) == Method.PUBLIC)
                {
                    data.put(m.getName(), new ExecuteMethod(m, this));
                }
            }
            String templateName = StringUtils.replace(this.getClass().getName(), ".", "/") + ".html";
            data.put("this", this);
            try
            {
                this.configuration.setTemplateLoader(new ClassTemplateLoader(this.getClass(), "/"));
                Template t = this.configuration.getTemplate(templateName, getLocale());
                StringWriter out = new StringWriter();
                t.process(data, out);
                body = out.toString();
            }
            catch (TemplateException e)
            {
                throw new NotificationException(e);
            }
            catch (IOException e)
            {
                throw new NotificationException(e);
            }

        }
        return body;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public abstract String getMessageSubject();

    public class ExecuteMethod implements TemplateMethodModelEx
    {

        private Method method;

        private Object parent;

        public ExecuteMethod(Method method, Object parent)
        {
            this.method = method;
            this.parent = parent;
        }

        /**
         * {@inheritDoc}
         */
        @SuppressWarnings("unchecked")
        @Override
        public Object exec(List arguments) throws TemplateModelException
        {
            try
            {
                if (arguments == null || arguments.size() == 0)
                {
                    return method.invoke(parent);
                }
                else
                {
                    List unwrappedArgs = new ArrayList();
                    for (Object o : arguments)
                    {
                        unwrappedArgs.add(((BeanModel) o).getWrappedObject());
                    }

                    if (unwrappedArgs.size() == 1)
                    {
                        return method.invoke(parent, unwrappedArgs.iterator().next());

                    }
                    else
                    {
                        return method.invoke(parent, unwrappedArgs.toArray(new Object[unwrappedArgs.size()]));
                    }
                }
            }
            catch (IllegalAccessException ex)
            {
                throw new TemplateModelException(ex);
            }
            catch (IllegalArgumentException ex)
            {
                throw new TemplateModelException(ex);
            }
            catch (InvocationTargetException ex)
            {
                throw new TemplateModelException(ex);
            }
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<MimeBodyPart> getAttachments()
    {
        return null;
    }

    /**
     * Sets the data.
     * @param data the data to set
     */
    public void setData(Map<String, Object> data)
    {
        this.data = data;
    }

}