package net.sourceforge.openutils.notifications.notifiers;

import java.util.List;
import java.util.Properties;

import javax.mail.Address;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.Message.RecipientType;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import net.sourceforge.openutils.notifications.Notification;
import net.sourceforge.openutils.notifications.NotificationException;
import net.sourceforge.openutils.notifications.messages.EmailNotification;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author molaschi
 * @version $Id: $
 */
public class EmailNotifier implements Notifier
{

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(EmailNotifier.class);

    private String mailHost;

    private String mailPort;

    private boolean async = false;

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean send(String from, List<String> toAddresses, List<String> ccAddresses, List<String> ccnAddresses,
        Notification notification) throws NotificationException
    {
        if (log.isDebugEnabled() && notification != null)
        {
            log.debug("Subject: {}", notification.getMessageSubject());
            log.debug("Body: {}", notification.getMessageBody());
        }

        Properties props = new Properties(); // System.getProperties(); should I try to use the system properties ?
        props.put("mail.smtp.host", this.mailHost);
        props.put("mail.smtp.port", this.mailPort);

        Session smtpSession = Session.getInstance(props);
        MimeMessage message = new MimeMessage(smtpSession);

        List<MimeBodyPart> attachments = ((EmailNotification) notification).getAttachments();
        try
        {
            message.addFrom(new Address[]{new InternetAddress(from) });
            message.setSubject(notification.getMessageSubject());
            if (attachments == null || attachments.size() == 0)
            {
                message.setContent(notification.getMessageBody(), "text/html; charset=UTF-8");
            }
            else
            {
                MimeMultipart multiPart = new MimeMultipart();

                MimeBodyPart text = new MimeBodyPart();
                text.setContent(notification.getMessageBody(), "text/html; charset=UTF-8");

                multiPart.addBodyPart(text);

                for (MimeBodyPart mbp : attachments)
                {
                    multiPart.addBodyPart(mbp);
                }

                message.setContent(multiPart);
            }
        }
        catch (MessagingException ex)
        {
            throw new NotificationException(ex);
        }

        for (String to : toAddresses)
        {
            try
            {
                message.setRecipient(RecipientType.TO, new InternetAddress(to));

                if (ccAddresses != null)
                {
                    for (String address : ccAddresses)
                    {
                        if (StringUtils.isNotEmpty(address))
                        {
                            message.addRecipient(RecipientType.CC, new InternetAddress(address));
                        }
                    }
                }
                if (ccnAddresses != null)
                {
                    for (String address : ccnAddresses)
                    {
                        if (StringUtils.isNotEmpty(address))
                        {
                            message.addRecipient(RecipientType.BCC, new InternetAddress(address));
                        }
                    }
                }
                Transport.send(message);
            }
            catch (MessagingException ex)
            {
                throw new NotificationException(ex);
            }

        }

        return true;
    }

    /**
     * Returns the mailHost.
     * @return the mailHost
     */
    public String getMailHost()
    {
        return mailHost;
    }

    /**
     * Sets the mailHost.
     * @param mailHost the mailHost to set
     */
    public void setMailHost(String mailHost)
    {
        this.mailHost = mailHost;
    }

    /**
     * Returns the mailPort.
     * @return the mailPort
     */
    public String getMailPort()
    {
        return mailPort;
    }

    /**
     * Sets the mailPort.
     * @param mailPort the mailPort to set
     */
    public void setMailPort(String mailPort)
    {
        this.mailPort = mailPort;
    }

    /**
     * Returns the async.
     * @return the async
     */
    public boolean isAsync()
    {
        return async;
    }

    /**
     * Sets the async.
     * @param async the async to set
     */
    public void setAsync(boolean async)
    {
        this.async = async;
    }

}