package net.sourceforge.openutils.notifications.services;

import java.util.Arrays;
import java.util.List;

import net.sourceforge.openutils.notifications.Notification;
import net.sourceforge.openutils.notifications.NotificationException;
import net.sourceforge.openutils.notifications.notifiers.Notifier;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;


/**
 * @author molaschi
 * @version $Id: $
 */
public class DefaultNotificationManagerImpl implements ApplicationContextAware, NotificationManager
{

    private ApplicationContext context;

    /**
     * {@inheritDoc}
     */
    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException
    {
        this.context = applicationContext;
    }

    /**
     * {@inheritDoc}
     */
    public boolean send(String from, List<String> toAddresses, List<String> ccAddresses, Notification notification)
        throws NotificationException
    {
        Notifier notifier = (Notifier) context.getBean(notification.getNotifierBeanName());
        return notifier.send(from, toAddresses, ccAddresses, null, notification);
    }

    /**
     * {@inheritDoc}
     */
    public boolean send(String from, List<String> toAddresses, List<String> ccAddresses, List<String> ccnAddresses,
        Notification notification) throws NotificationException
    {
        Notifier notifier = (Notifier) context.getBean(notification.getNotifierBeanName());
        return notifier.send(from, toAddresses, ccAddresses, ccnAddresses, notification);
    }

    /**
     * {@inheritDoc}
     */
    public boolean send(String from, String toAddress, List<String> ccAddresses, Notification notification)
        throws NotificationException
    {
        Notifier notifier = (Notifier) context.getBean(notification.getNotifierBeanName());
        return notifier.send(from, Arrays.asList(toAddress), ccAddresses, null, notification);
    }

    /**
     * {@inheritDoc}
     */
    public boolean send(String from, String toAddress, List<String> ccAddresses, List<String> ccnAddresses,
        Notification notification) throws NotificationException
    {
        Notifier notifier = (Notifier) context.getBean(notification.getNotifierBeanName());
        return notifier.send(from, Arrays.asList(toAddress), ccAddresses, ccnAddresses, notification);
    }

    /**
     * {@inheritDoc}
     */
    public boolean send(String from, String toAddress, Notification notification) throws NotificationException
    {
        Notifier notifier = (Notifier) context.getBean(notification.getNotifierBeanName());
        return notifier.send(from, Arrays.asList(toAddress), null, null, notification);
    }

    /**
     * {@inheritDoc}
     */
    public boolean send(String from, List<String> toAddresses, Notification notification) throws NotificationException
    {
        Notifier notifier = (Notifier) context.getBean(notification.getNotifierBeanName());
        return notifier.send(from, toAddresses, null, null, notification);
    }
}
